"""
Author: Michel Peltriaux
Organization: Struktur- und Genehmigungsdirektion Nord, Rhineland-Palatinate, Germany
Contact: ksp-servicestelle@sgdnord.rlp.de
Created on: 18.08.22

"""
from django.http import HttpRequest
from django.template.loader import render_to_string
from django.urls import reverse
from django.utils.html import format_html
from django.utils.translation import gettext_lazy as _

from compensation.filters.eco_account import EcoAccountTableFilter
from compensation.models import EcoAccount
from konova.utils.tables import TableRenderMixin, BaseTable

import django_tables2 as tables


class EcoAccountTable(BaseTable, TableRenderMixin):
    id = tables.Column(
        verbose_name=_("Identifier"),
        orderable=True,
        accessor="identifier",
    )
    t = tables.Column(
        verbose_name=_("Title"),
        orderable=True,
        accessor="title",
    )
    d = tables.Column(
        verbose_name=_("Parcel gmrkng"),
        orderable=True,
        accessor="geometry",
    )
    av = tables.Column(
        verbose_name=_("Available"),
        orderable=True,
        empty_values=[],
        attrs={
            "th": {
                "class": "w-20",
            }
        }
    )
    r = tables.Column(
        verbose_name=_("Recorded"),
        orderable=True,
        empty_values=[],
        accessor="recorded",
    )
    e = tables.Column(
        verbose_name=_("Editable"),
        orderable=True,
        empty_values=[],
        accessor="users",
    )
    lm = tables.Column(
        verbose_name=_("Last edit"),
        orderable=True,
        accessor="modified__timestamp",
    )

    class Meta(BaseTable.Meta):
        template_name = "django_tables2/bootstrap4.html"

    def __init__(self,  request: HttpRequest, *args, **kwargs):
        self.title = _("Eco Accounts")
        self.add_new_url = reverse("compensation:acc:new")
        qs = kwargs.get("queryset", None)
        self.filter = EcoAccountTableFilter(
            user=request.user,
            data=request.GET,
            queryset=qs,
        )
        kwargs["queryset"] = self.filter.qs
        super().__init__(request, *args, **kwargs)

    def render_id(self, value, record: EcoAccount):
        """ Renders the id column for an eco account

        Args:
            value (str): The identifier value
            record (EcoAccount): The eco account record

        Returns:

        """
        html = ""
        html += self.render_link(
            tooltip=_("Open {}").format(_("Eco-account")),
            href=reverse("compensation:acc:detail", args=(record.id,)),
            txt=value,
            new_tab=False,
        )
        return format_html(html)

    def render_av(self, value, record: EcoAccount):
        """ Renders the available column for an eco account

        Args:
            value (str): The identifier value
            record (EcoAccount): The eco account record

        Returns:

        """
        value_total, value_relative = record.get_available_rest()
        html = render_to_string("konova/widgets/progressbar.html", {"value": value_relative})
        return format_html(html)

    def render_d(self, value, record):
        """ Renders the parcel district column for a compensation

        Args:
            value (str): The geometry
            record (Compensation): The compensation record

        Returns:

        """
        parcels = value.get_underlying_parcels().values_list(
            "parcel_group__name",
            flat=True
        ).distinct()
        html = render_to_string(
            "table/gmrkng_col.html",
            {
                "entries": parcels
            }
        )
        return html

    def render_r(self, value, record: EcoAccount):
        """ Renders the recorded column for an eco account

        Args:
            value (str): The identifier value
            record (EcoAccount): The eco account record

        Returns:

        """
        html = ""
        checked = value is not None
        tooltip = _("Not recorded yet. Can not be used for deductions, yet.")
        if checked:
            on = value.get_timestamp_str_formatted()
            tooltip = _("Recorded on {} by {}").format(on, record.recorded.user)
        html += self.render_bookmark(
            tooltip=tooltip,
            icn_filled=checked,
        )
        return format_html(html)

    def render_e(self, value, record: EcoAccount):
        """ Renders the editable column for an eco account

        Args:
            value (str): The identifier value
            record (EcoAccount): The eco account record

        Returns:

        """
        html = ""
        # Do not use value in here, since value does use unprefetched 'users' manager, where record has already
        # prefetched users data
        has_access = record.is_shared_with(self.user)
        html += self.render_icn(
            tooltip=_("Full access granted") if has_access else _("Access not granted"),
            icn_class="fas fa-edit rlp-r-inv" if has_access else "far fa-edit",
        )
        return format_html(html)
